'**********************************************************
' C-Control Station
'
' Beispiel - einfache Alarmanlage
' an den Digitalports P1 bis P6 sind verschiedene Signalgeber
' angeschlossen:
'
' Etage 1
' P1  <-______/ ________/ ________/ ________/ ___ ... ___-> GND
'         Fenster10  Fenster11  Tuer10   Tuer11
'
' Etage 2
' P2  <-______/ ________/ ________/ ________/ ___ ... ___-> GND
'         Fenster20  Fenster21  Tuer20   Tuer21
'
' die Ports P1 und P2 ueberwachen reihengeschaltete Kontakte
' an Fenstern und Tueren auf zwei Etagen, ist ein Fenster oder
' eine Tuer offen, dann ist der Signalpegel an P1 bzw. P2 HIGH
'
' Etage 1                         Etage 2
' P3 <------ IR10 -------> GND    P4 <------ IR20 ------> GND
'        |-- IR11 --|                    |-- IR21 -- |
'        |-- IR12 --|                    |-- IR22 -- |
'            ...                             ...
'
' die Ports P3 und P4 ueberwachen Passiv-Infrarotsensoren mit
' Opencollector-Ausgang, befindet sich eine Person im Meldebereich,
' dann ist der Signalpegel an P3 bzw. P4 LOW
'
'
' P5 <-____/ ____-> GND         P6 ->------|>|-------> GND
'
' die Alarmanlage wird durch Schliessen eines Kontaktes an Port
' P5 geschaerft (z.B. im Schloss der Haupteingangstuer), die LED 1
' und eine zusaetzliche LED an Port P6 zeigen den Schaerfungszustand an
'
' nach Oeffnen des Kontakts an P5 (z.B. Aufschliessen der Eingangstuer)
' bleibt eine gewisse Zeit (MAX_DELAY in Sekunden) zum Entschaerfen
' der Anlage durch Eingabe eines Codes an den Funktionstasten der Station
'
' Der Code wird als 8-stellige Zahl zur Basis 4 eingegeben, die Tasten
' entsprechchen folgenden Ziffern F1=0, F2=1, F3=2, F4=3
'
' Beispiel:
'
'  Code-Kombination
'
'      F1 F2 F3 F4 F1 F2 F3 F4
'
'  =  01230123 (Basis 4)
'  =  0*4^7 + 1*4^6 + 2*4^5 + 3*4^4 + 0*4^3 + 1*4^2 + 2*4^1 + 3*4^0 (dezimal)
'  =           4096 +  2048 +   768         +    16 +     8 +     3 (dezimal)
'  =  6939 (dezimal)
'
' Bei Alarm schliesst Relais K1, und oeffnet Relais K2
'**********************************************************

' --- Definitionen ---

' die Relais
define K1 port[7]
define K2 port[8]

' die Folien-Funktionstasten
define F1 port[9]
define F2 port[10]
define F3 port[11]
define F4 port[12]

' die LEDs
define LED1 port[13]
define LED2 port[14]
define LED3 port[15]
define LED4 port[16]

' die Digitalports
define doors1 port[1]
define doors2 port[2]
define ir1 port[3]
define ir2 port[4]
define trigger port[5]
define armedLED port[6]


define code word
define countKey byte
define countRetry byte
define timeout byte

define VALID_CODE 6939  'entspricht Codesequenz F1-F2-F3-F4-F1-F2-F3-F4
define MAX_CODEDIGITS 8 '65535 verschiedene Codes
define MAX_CODERETRY 3  '
define MAX_DELAY 30


' nach Reset Code abfragen
armedLED = ON
LED1 = armedLED
gosub getcode

#loop
  if not trigger then gosub checkHouse else if armedLED then gosub getcode
  pause 10
goto loop

#checkHouse
  armedLED = ON
  LED1 = armedLED
  if doors1 or doors2 or not ir1 or not ir2 then goto alertandgetcode
return

#alertandgetcode
  gosub alert


' --- Codeabfrage ---

#getcode
  countRetry = 0
  timeout = (second + MAX_DELAY) mod 60
#retry
  code = 0
  countKey = 0
#codeloop
  if armedLED and (second = timeout) then gosub alert
  if not F1 then gosub handleF1
  if not F2 then gosub handleF2
  if not F3 then gosub handleF3
  if not F4 then gosub handleF4
  if MAX_CODEDIGITS = countKey then goto checkCode
goto codeloop

#handleF1
  code = code * 4
  goto releaseKey
#handleF2
  code = code * 4 + 1
  goto releaseKey
#handleF3
  code = code * 4 + 2
  goto releaseKey
#handleF4
  code = code * 4 + 3
  goto releaseKey
#releaseKey
  countKey = countKey + 1
  pause 5
  wait F1
  wait F2
  wait F3
  wait F4
return

#checkCode
  if code = VALID_CODE then goto shutUp
  beep 100, 5, 2
  beep 100, 5, 2
  beep 100, 5, 2
  countRetry = countRetry + 1
  if MAX_CODERETRY = countRetry then gosub alert
goto retry


' --- Alarm ---

#alert
  K1 = ON
  K2 = OFF
  beep 100,0,0
return

#shutUp
  K1 = OFF
  K2 = ON
  beep 0,0,0
  armedLED = OFF
  LED1 = armedLED
return

